import * as React from 'react';
import '../../style/trace-viewer.css';
import '../../style/trace-context-style.css';
import '../../style/output-components-style.css';
import '../../style/trace-explorer.css';
import '../../style/status-bar.css';
import { Layout } from 'react-grid-layout';
import { OutputDescriptor } from 'tsp-typescript-client/lib/models/output-descriptor';
import { Experiment } from 'tsp-typescript-client/lib/models/experiment';
import { TspClient } from 'tsp-typescript-client/lib/protocol/tsp-client';
import { TimeRange, TimeRangeString } from 'traceviewer-base/lib/utils/time-range';
import { OutputComponentStyle } from './utils/output-component-style';
import * as Messages from 'traceviewer-base/lib/message-manager';
import { TimeRangeUpdatePayload } from 'traceviewer-base/lib/signals/time-range-data-signal-payloads';
export interface TraceContextProps {
    tspClient: TspClient;
    experiment: Experiment;
    outputs: OutputDescriptor[];
    overviewDescriptor?: OutputDescriptor;
    markerCategoriesMap: Map<string, string[]>;
    markerSetId: string;
    onOutputRemove: (outputId: string) => void;
    onOverviewRemove: () => void;
    messageManager: Messages.MessageManager;
    addResizeHandler: (handler: () => void) => void;
    removeResizeHandler: (handler: () => void) => void;
    backgroundTheme: string;
    persistedState?: PersistedState;
}
export interface TraceContextState {
    timeOffset: bigint;
    currentRange: TimeRange;
    currentViewRange: TimeRange;
    currentTimeSelection: TimeRange | undefined;
    experiment: Experiment;
    traceIndexing: boolean;
    style: OutputComponentStyle;
    backgroundTheme: string;
    pinnedView: OutputDescriptor | undefined;
}
export interface PersistedState {
    outputs: OutputDescriptor[];
    currentRange: TimeRangeString;
    currentViewRange: TimeRangeString;
    currentTimeSelection?: TimeRangeString;
    unitControllerViewRange: {
        start: string;
        end: string;
    };
    storedTimescaleLayout: Layout[];
    storedNonTimescaleLayout: Layout[];
    pinnedView: OutputDescriptor | undefined;
    storedPinnedViewLayout: Layout | undefined;
    storedOverviewOutput: OutputDescriptor | undefined;
    storedOverviewLayout: Layout | undefined;
}
export declare class TraceContextComponent extends React.Component<TraceContextProps, TraceContextState> {
    private readonly INDEXING_RUNNING_STATUS;
    private readonly INDEXING_CLOSED_STATUS;
    private readonly INDEXING_STATUS_BAR_KEY;
    private readonly TIME_SELECTION_STATUS_BAR_KEY;
    private readonly OVERVIEW_OUTPUT_GRID_LAYOUT_ID;
    private readonly DEFAULT_COMPONENT_HEIGHT;
    private readonly DEFAULT_COMPONENT_ROWHEIGHT;
    private readonly DEFAULT_OVERVIEW_HEIGHT;
    private readonly DEFAULT_OVERVIEW_ROWHEIGHT;
    private readonly DEFAULT_COMPONENT_LEFT;
    private readonly SCROLLBAR_PADDING;
    private readonly DEFAULT_CHART_OFFSET;
    private readonly MIN_COMPONENT_HEIGHT;
    private chartPersistedState;
    private unitController;
    private historyHandler;
    private tooltipComponent;
    private tooltipXYComponent;
    private traceContextContainer;
    private _storedTimescaleLayout;
    private _storedNonTimescaleLayout;
    private _storedPinnedViewLayout;
    private _storedOverviewLayout;
    protected widgetResizeHandlers: (() => void)[];
    protected readonly addWidgetResizeHandler: (h: () => void) => void;
    protected readonly removeWidgetResizeHandler: (h: () => void) => void;
    private onBackgroundThemeChange;
    private onUpdateZoom;
    private onResetZoom;
    private onPinView;
    private onUnPinView;
    constructor(props: TraceContextProps);
    private doHandleBackgroundThemeChange;
    private initialize;
    private updateTrace;
    private sleep;
    componentDidMount(): void;
    componentWillUnmount(): void;
    private subscribeToEvents;
    private unsubscribeToEvents;
    componentDidUpdate(prevProps: TraceContextProps, prevState: TraceContextState): Promise<void>;
    private scrollToBottom;
    private scrollToPinnedView;
    private scrollToUnPinnedView;
    private doHandleUpdateZoomSignal;
    private doHandleResetZoomSignal;
    private zoomButton;
    private onResize;
    private setChartOffset;
    private handleTimeSelectionChange;
    private handleViewRangeChange;
    private emitTimeRangeData;
    private onContextMenu;
    private onLayoutChange;
    render(): JSX.Element;
    private onKeyDown;
    private onKeyUp;
    private zoomToSelected;
    private renderOutputs;
    private renderGridLayout;
    get persistedState(): PersistedState;
    private renderPlaceHolder;
    private undoHistory;
    private redoHistory;
    private updateHistory;
    onRequestToUpdateSelectionRange: (payload: TimeRangeUpdatePayload) => void;
    private doHandleUnPinView;
    private doHandlePinView;
    private generateGridLayout;
}
//# sourceMappingURL=trace-context-component.d.ts.map