"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TimeRangeDataMap = void 0;
class TimeRangeDataMap {
    constructor() {
        this.updateViewRange = (payload) => {
            const { experimentUUID: UUID, timeRange } = payload;
            const update = {
                UUID,
                viewRange: timeRange
            };
            this.updateExperimentTimeRangeData(update);
        };
        this.updateSelectionRange = (payload) => {
            const { experimentUUID: UUID, timeRange } = payload;
            const update = {
                UUID,
                selectionRange: timeRange
            };
            this.updateExperimentTimeRangeData(update);
        };
        this.updateAbsoluteRange = (experiment) => {
            if (!experiment) {
                return;
            }
            const { UUID, start, end } = experiment;
            const update = {
                UUID,
                absoluteRange: {
                    start,
                    end
                }
            };
            this.updateExperimentTimeRangeData(update);
        };
        /**
         * Updates the data stored in experimentDataMap.  Works similar to setState() where
         * you only input the data to change and the existing values persist.
         * @param data Partial data of Experiment Time Range Data.
         */
        this.updateExperimentTimeRangeData = (data) => {
            var _a;
            const map = this._experimentDataMap;
            const id = data.UUID;
            const existingData = map.get(id) || {};
            const newData = Object.assign(Object.assign({}, existingData), data);
            map.set(id, newData);
            // If the experiment is currently displayed, we need to render it
            if (id === ((_a = this._activeData) === null || _a === void 0 ? void 0 : _a.UUID)) {
                this.setActiveExperiment(newData);
            }
        };
        this.delete = (experiment) => {
            const id = typeof experiment === 'string' ? experiment : experiment.UUID;
            this._experimentDataMap.delete(id);
        };
        this._experimentDataMap = new Map();
    }
    setActiveExperiment(data) {
        this._activeData = data;
    }
    get(UUID) {
        return this._experimentDataMap.get(UUID);
    }
    set(data) {
        this._experimentDataMap.set(data.UUID, data);
    }
    clear() {
        this._experimentDataMap.clear();
    }
    get activeData() {
        return this._activeData;
    }
    get experimentDataMap() {
        return this._experimentDataMap;
    }
}
exports.TimeRangeDataMap = TimeRangeDataMap;
//# sourceMappingURL=time-range-data-map.js.map